import java.util.Arrays;

import at.jku.ssw.Sorter;
import at.jku.ssw.BubbleSort;
import at.jku.ssw.SelectionSort;
import at.jku.ssw.InsertionSort;
import at.jku.ssw.JavaSort;
import at.jku.ssw.QuickSort;
import at.jku.ssw.MergeSort;

import at.jku.students.HeapSort;

public class SortTest {
  
  public static void main(String[] args) {
    test1();
    test2();
  }
  
  static void test1() {
    Sorter heapSort = new HeapSort();
    Integer[] values = new Integer[] { 10, 3, 1, 7, 6, 5, 4, 2, 9, 8 };
    heapSort.sort(values);
    for (int x: values) {
      Out.print(" " + x);
    }
  }

  static void test2() {
    Comparable[] data = generateIntegers(1000, Order.RANDOM);

    Sorter[] sorters = new Sorter[] { 
       new BubbleSort(), new SelectionSort(),
       new InsertionSort(), new MergeSort(),
       new JavaSort(), new QuickSort(),
       new HeapSort() };

     Measurement[] measurements = new Measurement[sorters.length];
     for (int i = 0; i < sorters.length; i++) {
       measurements[i] = new Measurement(sorters[i]);
     }

    final int REPEAT = 50;
    for (Measurement m: measurements) {
      m.timer = Timer.start();
      for (int i = 0; i < REPEAT; i++) {
        m.timer.pause();
        m.sortedData = data.clone();
        m.timer.continu();
        m.sorter.sort(m.sortedData);
      }
      m.timer.stop();
    }
    Arrays.sort(measurements);
    
    new EvaluationUI(measurements, data).show();
  }
  
  public static enum Order { ASCENDING, DESCENDING, RANDOM };
  
  private static Comparable[] generateIntegers(int cnt, Order order) {
    Integer[] arr = new Integer[cnt];
    for (int i = 0; i < cnt; i++) {
      switch (order) {
        case ASCENDING:
          arr[i] = i;
          break;
        case DESCENDING:
          arr[i] = cnt - i;
          break;
        case RANDOM:
          arr[i] = (int) (Math.random() * cnt);
          break;
      }
    }
    return arr;
  }

}